@@ -579,22 +579,31 @@ create_subsurface_tree(struct client *client, struct wl_surface **surfs,
 
 	case 11:
 		SUB_LINK(10, 2);
+		/* fallthrough */
 	case 10:
 		SUB_LINK(9, 2);
+		/* fallthrough */
 	case 9:
 		SUB_LINK(8, 6);
+		/* fallthrough */
 	case 8:
 		SUB_LINK(7, 6);
+		/* fallthrough */
 	case 7:
 		SUB_LINK(6, 2);
+		/* fallthrough */
 	case 6:
 		SUB_LINK(5, 1);
+		/* fallthrough */
 	case 5:
 		SUB_LINK(4, 3);
+		/* fallthrough */
 	case 4:
 		SUB_LINK(3, 1);
+		/* fallthrough */
 	case 3:
 		SUB_LINK(2, 0);
+		/* fallthrough */
 	case 2:
 		SUB_LINK(1, 0);
 
@@ -685,11 +694,11 @@ destroy_permu_object(struct wl_surface **surfs,
 	int h = (i + 1) / 2;
 
 	if (i & 1) {
-		fprintf(stderr, " [sub  %2d]", h);
+		testlog(" [sub  %2d]", h);
 		wl_subsurface_destroy(subs[h]);
 		subs[h] = NULL;
 	} else {
-		fprintf(stderr, " [surf %2d]", h);
+		testlog(" [surf %2d]", h);
 		wl_surface_destroy(surfs[h]);
 		surfs[h] = NULL;
 	}
@@ -727,15 +736,15 @@ TEST(test_subsurface_destroy_permutations)
 
 		create_subsurface_tree(client, surfs, subs, test_size);
 
-		fprintf(stderr, "permu");
+		testlog("permu");
 
 		for (i = 0; i < NSTEPS; i++)
-			fprintf(stderr, " %2d", per.cnt[i]);
+			testlog(" %2d", per.cnt[i]);
 
 		for (i = 0; i < NSTEPS; i++)
 			destroy_permu_object(surfs, subs, per.cnt[i]);
 
-		fprintf(stderr, "\n");
+		testlog("\n");
 		client_roundtrip(client);
 
 		destroy_subsurface_tree(surfs, subs, test_size);
@@ -743,5 +752,5 @@ TEST(test_subsurface_destroy_permutations)
 	}
 
 	client_roundtrip(client);
-	fprintf(stderr, "tried %d destroy permutations\n", counter);
+	testlog("tried %d destroy permutations\n", counter);
 }
