@@ -1,30 +1,36 @@
 /*
  * Copyright © 2012 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
+#include "config.h"
+
 #include <string.h>
+#include <stdint.h>
 #include <stdio.h>
-#include <linux/input.h>
+
 #include "weston-test-client-helper.h"
-#include "text-client-protocol.h"
+#include "text-input-unstable-v1-client-protocol.h"
 
 struct text_input_state {
 	int activated;
@@ -33,7 +39,7 @@ struct text_input_state {
 
 static void
 text_input_commit_string(void *data,
-			 struct wl_text_input *text_input,
+			 struct zwp_text_input_v1 *text_input,
 			 uint32_t serial,
 			 const char *text)
 {
@@ -41,7 +47,7 @@ text_input_commit_string(void *data,
 
 static void
 text_input_preedit_string(void *data,
-			  struct wl_text_input *text_input,
+			  struct zwp_text_input_v1 *text_input,
 			  uint32_t serial,
 			  const char *text,
 			  const char *commit)
@@ -50,7 +56,7 @@ text_input_preedit_string(void *data,
 
 static void
 text_input_delete_surrounding_text(void *data,
-				   struct wl_text_input *text_input,
+				   struct zwp_text_input_v1 *text_input,
 				   int32_t index,
 				   uint32_t length)
 {
@@ -58,7 +64,7 @@ text_input_delete_surrounding_text(void *data,
 
 static void
 text_input_cursor_position(void *data,
-			   struct wl_text_input *text_input,
+			   struct zwp_text_input_v1 *text_input,
 			   int32_t index,
 			   int32_t anchor)
 {
@@ -66,7 +72,7 @@ text_input_cursor_position(void *data,
 
 static void
 text_input_preedit_styling(void *data,
-			   struct wl_text_input *text_input,
+			   struct zwp_text_input_v1 *text_input,
 			   uint32_t index,
 			   uint32_t length,
 			   uint32_t style)
@@ -75,21 +81,21 @@ text_input_preedit_styling(void *data,
 
 static void
 text_input_preedit_cursor(void *data,
-			  struct wl_text_input *text_input,
+			  struct zwp_text_input_v1 *text_input,
 			  int32_t index)
 {
 }
 
 static void
 text_input_modifiers_map(void *data,
-			 struct wl_text_input *text_input,
+			 struct zwp_text_input_v1 *text_input,
 			 struct wl_array *map)
 {
 }
 
 static void
 text_input_keysym(void *data,
-		  struct wl_text_input *text_input,
+		  struct zwp_text_input_v1 *text_input,
 		  uint32_t serial,
 		  uint32_t time,
 		  uint32_t sym,
@@ -100,20 +106,20 @@ text_input_keysym(void *data,
 
 static void
 text_input_enter(void *data,
-		 struct wl_text_input *text_input,
+		 struct zwp_text_input_v1 *text_input,
 		 struct wl_surface *surface)
 
 {
 	struct text_input_state *state = data;
 
-	fprintf(stderr, "%s\n", __FUNCTION__);
+	testlog("%s\n", __FUNCTION__);
 
 	state->activated += 1;
 }
 
 static void
 text_input_leave(void *data,
-		 struct wl_text_input *text_input)
+		 struct zwp_text_input_v1 *text_input)
 {
 	struct text_input_state *state = data;
 
@@ -122,14 +128,14 @@ text_input_leave(void *data,
 
 static void
 text_input_input_panel_state(void *data,
-			     struct wl_text_input *text_input,
+			     struct zwp_text_input_v1 *text_input,
 			     uint32_t state)
 {
 }
 
 static void
 text_input_language(void *data,
-		    struct wl_text_input *text_input,
+		    struct zwp_text_input_v1 *text_input,
 		    uint32_t serial,
 		    const char *language)
 {
@@ -137,13 +143,13 @@ text_input_language(void *data,
 
 static void
 text_input_text_direction(void *data,
-			  struct wl_text_input *text_input,
+			  struct zwp_text_input_v1 *text_input,
 			  uint32_t serial,
 			  uint32_t direction)
 {
 }
 
-static const struct wl_text_input_listener text_input_listener = {
+static const struct zwp_text_input_v1_listener text_input_listener = {
 	text_input_enter,
 	text_input_leave,
 	text_input_modifiers_map,
@@ -163,52 +169,54 @@ TEST(text_test)
 {
 	struct client *client;
 	struct global *global;
-	struct wl_text_input_manager *factory;
-	struct wl_text_input *text_input;
+	struct zwp_text_input_manager_v1 *factory;
+	struct zwp_text_input_v1 *text_input;
 	struct text_input_state state;
 
-	client = client_create(100, 100, 100, 100);
+	client = create_client_and_test_surface(100, 100, 100, 100);
 	assert(client);
 
 	factory = NULL;
 	wl_list_for_each(global, &client->global_list, link) {
-		if (strcmp(global->interface, "wl_text_input_manager") == 0)
+		if (strcmp(global->interface, "zwp_text_input_manager_v1") == 0)
 			factory = wl_registry_bind(client->wl_registry,
 						   global->name,
-						   &wl_text_input_manager_interface, 1);
+						   &zwp_text_input_manager_v1_interface, 1);
 	}
 
 	assert(factory);
 
 	memset(&state, 0, sizeof state);
-	text_input = wl_text_input_manager_create_text_input(factory);
-	wl_text_input_add_listener(text_input, &text_input_listener, &state);
+	text_input = zwp_text_input_manager_v1_create_text_input(factory);
+	zwp_text_input_v1_add_listener(text_input,
+				       &text_input_listener,
+				       &state);
 
 	/* Make sure our test surface has keyboard focus. */
-	wl_test_activate_surface(client->test->wl_test,
+	weston_test_activate_surface(client->test->weston_test,
 				 client->surface->wl_surface);
 	client_roundtrip(client);
 	assert(client->input->keyboard->focus == client->surface);
 
 	/* Activate test model and make sure we get enter event. */
-	wl_text_input_activate(text_input, client->input->wl_seat,
-			       client->surface->wl_surface);
+	zwp_text_input_v1_activate(text_input, client->input->wl_seat,
+				   client->surface->wl_surface);
 	client_roundtrip(client);
 	assert(state.activated == 1 && state.deactivated == 0);
 
 	/* Deactivate test model and make sure we get leave event. */
-	wl_text_input_deactivate(text_input, client->input->wl_seat);
+	zwp_text_input_v1_deactivate(text_input, client->input->wl_seat);
 	client_roundtrip(client);
 	assert(state.activated == 1 && state.deactivated == 1);
 
 	/* Activate test model again. */
-	wl_text_input_activate(text_input, client->input->wl_seat,
-			       client->surface->wl_surface);
+	zwp_text_input_v1_activate(text_input, client->input->wl_seat,
+				   client->surface->wl_surface);
 	client_roundtrip(client);
 	assert(state.activated == 2 && state.deactivated == 1);
 
 	/* Take keyboard focus away and verify we get leave event. */
-	wl_test_activate_surface(client->test->wl_test, NULL);
+	weston_test_activate_surface(client->test->weston_test, NULL);
 	client_roundtrip(client);
 	assert(state.activated == 2 && state.deactivated == 2);
 }
