@@ -31,54 +31,55 @@
 
 #include <stdlib.h>
 
+#include <wayland-util.h>
 #include "shared/helpers.h"
 
 #ifdef NDEBUG
 #error "Tests must not be built with NDEBUG defined, they rely on assert()."
 #endif
 
-struct weston_test {
+struct weston_test_entry {
 	const char *name;
 	void (*run)(void *);
 	const void *table_data;
 	size_t element_size;
 	int n_elements;
-	int must_fail;
 } __attribute__ ((aligned (32)));
 
-#define TEST_BEGIN(name, arg)					\
+#define TEST_BEGIN(name, arg)						\
 	static void name(arg)
 
-#define TEST_COMMON(func, name, ret, data, size, n_elem)	\
-	static void func(void *);				\
-								\
-	const struct weston_test test##name			\
-		__attribute__ ((section ("test_section"))) =	\
-	{							\
-		#name, func, data, size, n_elem, ret		\
+#define TEST_COMMON(func, name, data, size, n_elem)			\
+	static void func(void *);					\
+									\
+	const struct weston_test_entry test##name			\
+		__attribute__ ((used, section ("test_section"))) =	\
+	{								\
+		#name, func, data, size, n_elem				\
 	};
 
-#define NO_ARG_TEST(name, ret)					\
-	TEST_COMMON(wrap##name, name, ret, NULL, 0, 1)		\
-	static void name(void);					\
-	static void wrap##name(void *data)			\
-	{							\
-		(void) data;					\
-		name();						\
-	}							\
-								\
+#define NO_ARG_TEST(name)						\
+	TEST_COMMON(wrap##name, name, NULL, 0, 1)			\
+	static void name(void);						\
+	static void wrap##name(void *data)				\
+	{								\
+		(void) data;						\
+		name();							\
+	}								\
+									\
 	TEST_BEGIN(name, void)
 
-#define ARG_TEST(name, ret, test_data)				\
-	TEST_COMMON(name, name, ret, test_data,			\
-		    sizeof(test_data[0]),			\
-		    ARRAY_LENGTH(test_data))			\
-	TEST_BEGIN(name, void *data)				\
+#define ARG_TEST(name, test_data)					\
+	TEST_COMMON(name, name, test_data,				\
+		    sizeof(test_data[0]),				\
+		    ARRAY_LENGTH(test_data))				\
+	TEST_BEGIN(name, void *data)					\
 
-#define TEST(name) NO_ARG_TEST(name, 0)
-#define FAIL_TEST(name) NO_ARG_TEST(name, 1)
-#define TEST_P(name, data) ARG_TEST(name, 0, data)
-#define FAIL_TEST_P(name, data) ARG_TEST(name, 1, data)
+#define TEST(name) NO_ARG_TEST(name)
+#define TEST_P(name, data) ARG_TEST(name, data)
+
+void
+testlog(const char *fmt, ...) WL_PRINTF(1, 2);
 
 /**
  * Get the test name string with counter
